#' Create Test Matrix from Excel File
#'
#' This function reads an Excel file containing Physchem and off-target data,
#' processes the data, and generates a test matrix compatible with the model.
#'
#' @param file_path Character. Path to the Excel file.
#' @return A matrix containing processed test data for prediction.
#' @import openxlsx
#' @examples
#' testX <- create_testX(system.file("extdata", 
#'                    "compound_template_example.xlsx", 
#'                     package = "predDIKI"))
#' @export
create_testX <- function(file_path) {
	# Load predictor data from package files
	finalPredictors2use <- readRDS(system.file("extdata", "finalPredictors2use.rds", package = "predDIKI"))
	
	# Read sheets from the input Excel file
	testData <- list(
			Physchem = openxlsx::read.xlsx(file_path, sheet = 1),
			offTargets = openxlsx::read.xlsx(file_path, sheet = 2)
	)
	
	geneNames2consider <- colnames(finalPredictors2use)[-c(1:4)]
	offPart <- list()
	
	# Process off-target data to create binary matrix
	for (iComp in unique(testData$offTargets$Compound)) {
		tmp <- rep(0, length(geneNames2consider))
		subOff <- testData$offTargets[testData$offTargets$Compound == iComp, ]
		tmp[which(geneNames2consider %in% subOff$geneName)] <- 1
		offPart <- append(offPart, list(tmp))
	}
	
	# Combine Physchem data with binary off-target data
	testX <- data.frame(testData$Physchem[, -1], do.call("rbind", offPart))
	colnames(testX) <- colnames(finalPredictors2use)
	rownames(testX) <- testData$Physchem$Compound
	return(as.matrix(testX))
}
