#' Predict Nephrotoxicity and Generate Plot
#'
#' This function takes a test matrix and makes nephrotoxicity predictions using pre-trained models.
#' It returns a table with predicted classes and probabilities and a plot.
#'
#' @param testX Matrix. The test data matrix generated by \code{\link{create_testX}}.
#' @return A list containing two elements:
#' \describe{
#'   \item{finalTable}{Data frame with compound names, predicted probabilities, and predicted classes.}
#'   \item{finalPlot}{A ggplot object showing probability distributions and classification regions.}
#' }
#' @import ggplot2
#' @import ggrepel
#' @import ggforce
#' @import glmnet
#' @import nnet
#' @import e1071
#' @examples
#' testX <- create_testX(system.file("extdata", "compound_template_example.xlsx", package = "predDIKI"))
#' results <- predict_nephrotoxicity(testX)
#' results$finalTable
#' results$finalPlot
#' @export
predict_nephrotoxicity <- function(testX) {

	# Load necessary data files from package
	fittedModels <- readRDS(system.file("extdata", "fittedModels.rds", package = "predDIKI"))
	bestThreshold <- readRDS(system.file("extdata", "bestThreshold.rds", package = "predDIKI"))
	predTrain <- readRDS(system.file("extdata", "predTrain.rds", package = "predDIKI"))
	
	# Extract models
	fitRLR <- fittedModels$fitRLR
	fitNN <- fittedModels$fitNN
	fitSVM <- fittedModels$fitSVM
	fitCombined <- fittedModels$fitCombined
	
	# Generate predictions using three models
	predRLR <- predict(fitRLR, type = "response", newx = testX)
	predNN <- predict(fitNN, newdata = testX)
	predSVM <- attributes(predict(fitSVM, newdata = data.frame(testX), probability = TRUE))$probabilities[, 1]
	
	# Combine model predictions
	combinedTestPreds <- data.frame(RLR = predRLR, NN = predNN, SVM = predSVM)
	
	# Predict final data using combined model
	predictedFinal <- predict(fitCombined, type = "response", newx = as.matrix(combinedTestPreds))
	
	# Generate the prediction table
	predTest <- data.frame(
			Compound = rownames(testX),
			Class = "Test",
			prob = predictedFinal[, 1],
			predictedClass = ifelse(predictedFinal[, 1] > bestThreshold$bestCut["RLR,NN,SVM"], "nephrotoxic", "non-nephrotoxic")
	)
	
	# Prepare data for plotting
	data2plot <- rbind(predTrain, predTest[, 1:3])
	cutpoint2add <- bestThreshold$bestCut["RLR,NN,SVM"]
	data2plot$Class <- factor(data2plot$Class, levels = c("Test",  "non-nephrotoxic", "nephrotoxic"))
	
	# Generate plot
	finalPlot <- ggplot() +
			geom_rect(aes(xmin = -Inf, xmax = Inf, ymin = 0, ymax = cutpoint2add),
					fill = "chartreuse4", alpha = 0.05) +
			geom_rect(aes(xmin = -Inf, xmax = Inf, ymin = cutpoint2add, ymax = 1),
					fill = "red1", alpha = 0.05) +
			geom_point(data = data2plot[data2plot$Class == "Test", ], aes(x = Class, y = prob, color = Class)) +
			geom_violin(data = data2plot[data2plot$Class != "Test", ], aes(x = Class, y = prob, color = Class, fill = Class)) +
			geom_sina(data = data2plot[data2plot$Class != "Test", ], aes(x = Class, y = prob, color = Class), alpha = 0.5) +
			geom_text_repel(data = data2plot[data2plot$Class == "Test", ], aes(x = Class, y = prob, color = Class, label = Compound),
					fontface = "bold", max.overlaps = Inf) +
			scale_color_manual(name = "Class", values = c("non-nephrotoxic" = "chartreuse4", "nephrotoxic" = "red1", "Test" = "#4a90e2")) +
			scale_fill_manual(name = "Class", values = c("non-nephrotoxic" = ggplot2::alpha("chartreuse4", 0.15), "nephrotoxic" = ggplot2::alpha("red1", 0.15),
							"Test" = ggplot2::alpha("#4a90e2", 0.15))) +
			theme_bw(base_size = 18) +
			geom_hline(yintercept = cutpoint2add, color = "gray", linetype = "dashed") +
			guides(color = "none", fill = "none") +
			xlab("Class") +
			ylab("Probability of nephrotoxicity") +
			theme(axis.text.x = element_text(face = "bold"),
					axis.text.y = element_text(face = "bold"))
	
	# Format final table
	finalTable <- predTest[,-2]
	colnames(finalTable)[2:3] <- c("Probability of nephrotoxicity", "Predicted Class")
	
	return(list(finalTable = finalTable, finalPlot = finalPlot))
}
