# Define UI
ui <- shiny::fluidPage(
		
		# Set the browser tab title
		shiny::tags$head(
				shiny::tags$title("pred-DIKI"),  # This sets the browser tab title
				shiny::tags$style(HTML("
										.well {
										background-color: #e0dccc !important;
										border: none;
										}
										.well img {
										background-color: #e0dccc;
										display: block;
										width: 100%;  /* Make the logo responsive to the sidebar width */
										height: auto;  /* Maintain aspect ratio */
										}
										"))
		),
		
		# App title with stylish subtitle displayed within the app
		shiny::titlePanel(
				shiny::tags$div(
						shiny::h1("pred-DIKI", style = "font-family: 'Arial Black', Gadget, sans-serif; font-size: 48px; color: #333; margin-bottom: 5px; font-weight: bold;"),
						shiny::h4("AI-powered early-stage Drug-Induced Kidney Injury Prediction", style = "font-family: 'Georgia', serif; font-size: 20px; color: #555; font-style: italic;")
				)
		),
		
		# Sidebar layout with file input and button
		shiny::sidebarLayout(
				shiny::sidebarPanel(
						
						# Download Template button
						shiny::downloadButton("downloadTemplate", "Download Template", style = "margin-top: 10px;"),
						shiny::tags$p("Download template Excel file to enter input data"),
						
						# File input to select Excel file
						shiny::fileInput("excel_file", "Upload Excel File", 
								accept = c(".xlsx"),
								buttonLabel = "Browse...",
								placeholder = "No file selected"),
						
						# Action button to initiate prediction
						shiny::actionButton("predict_btn", "Predict DIKI"),
						
						# Logo image below the Predict button
						shiny::tags$div(
								shiny::tags$img(src = "logo_kidney_2use.png", height = "300px", align = "center"),
								style = "text-align: center; padding-top: 20px;"
						),
						width = 3
				),
				
				# Main panel to display outputs
				shiny::mainPanel(
						# Output for plot with increased height
						shiny::plotOutput("predictionPlot", height = "500px"),
						shiny::uiOutput("plotDownloadUI"),  # Conditional download button for plot
						shiny::br(), shiny::br(),
						
						# Output for table
						shiny::tableOutput("predictionTable"),
						shiny::uiOutput("tableDownloadUI")  # Conditional download button for table
				)
		)
)

# Define server logic
server <- function(input, output, session) {
	
	# Download handler for the template file
	output$downloadTemplate <- shiny::downloadHandler(
			filename = function() {
				"compound_template.xlsx"
			},
			content = function(file) {
				# Copy the template file from the package's extdata folder to the download location
				file.copy(system.file("extdata", "compound_template.xlsx", package = "predDIKI"), file)
			}
	)
	
	# Reactive expression to read and process the Excel file
	testX <- shiny::reactive({
				shiny::req(input$excel_file)  # Ensure file is uploaded
				create_testX(input$excel_file$datapath)  # Use create_testX to process data
			})
	
	# Reactive expression to generate predictions and plot when the button is clicked
	predictionResults <- shiny::eventReactive(input$predict_btn, {
				shiny::req(testX())
				predict_nephrotoxicity(testX())
			})
	
	# Render the plot
	output$predictionPlot <- shiny::renderPlot({
				shiny::req(predictionResults())
				predictionResults()$finalPlot
			})
	
	# Render the prediction table
	output$predictionTable <- shiny::renderTable({
				shiny::req(predictionResults())
				predictionResults()$finalTable
			}, striped = TRUE, hover = TRUE, bordered = TRUE)
	
	# Conditional download button for plot (shown only when predictionResults is available)
	output$plotDownloadUI <- shiny::renderUI({
				shiny::req(predictionResults())
				shiny::downloadButton("downloadPlot", "Download Plot as PDF")
			})
	
	# Conditional download button for table (shown only when predictionResults is available)
	output$tableDownloadUI <- shiny::renderUI({
				shiny::req(predictionResults())
				shiny::downloadButton("downloadTable", "Download Table as Excel")
			})
	
	# Download handler for plot as PDF with wider dimensions
	output$downloadPlot <- shiny::downloadHandler(
			filename = function() {
				paste("plot_", Sys.Date(), ".pdf", sep = "")
			},
			content = function(file) {
				pdf(file, width = 10, height = 6)  # Wider PDF output dimensions
				print(predictionResults()$finalPlot)
				dev.off()
			}
	)
	
	# Download handler for table as Excel
	output$downloadTable <- shiny::downloadHandler(
			filename = function() {
				paste("table_", Sys.Date(), ".xlsx", sep = "")
			},
			content = function(file) {
				openxlsx::write.xlsx(predictionResults()$finalTable, file)
			}
	)
}

# Run the application
shiny::shinyApp(ui = ui, server = server)
